import fs from 'fs';
import { Readable } from 'stream';
import { resolvePath } from '@loaders.gl/loader-utils';
import { decompressReadStream } from "./stream-utils.node.js";
const isBoolean = x => typeof x === 'boolean';
const isFunction = x => typeof x === 'function';
const isObject = x => x !== null && typeof x === 'object';
const isReadableNodeStream = x => isObject(x) && isFunction(x.read) && isFunction(x.pipe) && isBoolean(x.readable);
export async function fetchNode(url, options) {
  const FILE_PROTOCOL_REGEX = /^file:\/\//;
  url.replace(FILE_PROTOCOL_REGEX, '/');
  let noqueryUrl = url.split('?')[0];
  noqueryUrl = resolvePath(noqueryUrl);
  const responseHeaders = new Headers();
  if (url.endsWith('.gz')) {
    responseHeaders['content-encoding'] = 'gzip';
  }
  if (url.endsWith('.br')) {
    responseHeaders['content-encoding'] = 'br';
  }
  try {
    const body = await new Promise((resolve, reject) => {
      const stream = fs.createReadStream(noqueryUrl, {
        encoding: null
      });
      stream.once('readable', () => resolve(stream));
      stream.on('error', error => reject(error));
    });
    let bodyStream = body;
    if (isReadableNodeStream(body)) {
      bodyStream = decompressReadStream(body, responseHeaders);
    } else if (typeof body === 'string') {
      bodyStream = Readable.from([new TextEncoder().encode(body)]);
    } else {
      bodyStream = Readable.from([body || new ArrayBuffer(0)]);
    }
    const status = 200;
    const statusText = 'OK';
    const headers = getHeadersForFile(noqueryUrl);
    const response = new Response(bodyStream, {
      headers,
      status,
      statusText
    });
    Object.defineProperty(response, 'url', {
      value: url
    });
    return response;
  } catch (error) {
    const errorMessage = error.message;
    const status = 400;
    const statusText = errorMessage;
    const headers = {};
    const response = new Response(errorMessage, {
      headers,
      status,
      statusText
    });
    Object.defineProperty(response, 'url', {
      value: url
    });
    return response;
  }
}
function getHeadersForFile(noqueryUrl) {
  const headers = {};
  if (!headers['content-length']) {
    const stats = fs.statSync(noqueryUrl);
    headers['content-length'] = stats.size;
  }
  if (noqueryUrl.endsWith('.gz')) {
    noqueryUrl = noqueryUrl.slice(0, -3);
    headers['content-encoding'] = 'gzip';
  }
  return new Headers(headers);
}
//# sourceMappingURL=fetch-node.js.map