import zlib from 'zlib';
const isArrayBuffer = x => x && x instanceof ArrayBuffer;
const isBuffer = x => x && x instanceof Buffer;
export function decompressReadStream(readStream, headers) {
  switch (headers === null || headers === void 0 ? void 0 : headers.get('content-encoding')) {
    case 'br':
      return readStream.pipe(zlib.createBrotliDecompress());
    case 'gzip':
      return readStream.pipe(zlib.createGunzip());
    case 'deflate':
      return readStream.pipe(zlib.createDeflate());
    default:
      return readStream;
  }
}
export async function concatenateReadStream(readStream) {
  const arrayBufferChunks = [];
  return await new Promise((resolve, reject) => {
    readStream.on('error', error => reject(error));
    readStream.on('readable', () => readStream.read());
    readStream.on('data', chunk => {
      if (typeof chunk === 'string') {
        reject(new Error('Read stream not binary'));
      }
      arrayBufferChunks.push(toArrayBuffer(chunk));
    });
    readStream.on('end', () => {
      const arrayBuffer = concatenateArrayBuffers(arrayBufferChunks);
      resolve(arrayBuffer);
    });
  });
}
export function concatenateArrayBuffers(sources) {
  const sourceArrays = sources.map(source2 => source2 instanceof ArrayBuffer ? new Uint8Array(source2) : source2);
  const byteLength = sourceArrays.reduce((length, typedArray) => length + typedArray.byteLength, 0);
  const result = new Uint8Array(byteLength);
  let offset = 0;
  for (const sourceArray of sourceArrays) {
    result.set(sourceArray, offset);
    offset += sourceArray.byteLength;
  }
  return result.buffer;
}
export function toArrayBuffer(data) {
  if (isArrayBuffer(data)) {
    return data;
  }
  if (isBuffer(data)) {
    const typedArray = new Uint8Array(data);
    return typedArray.buffer;
  }
  if (ArrayBuffer.isView(data)) {
    return data.buffer;
  }
  if (typeof data === 'string') {
    const text = data;
    const uint8Array = new TextEncoder().encode(text);
    return uint8Array.buffer;
  }
  if (data && typeof data === 'object' && data._toArrayBuffer) {
    return data._toArrayBuffer();
  }
  throw new Error(`toArrayBuffer(${JSON.stringify(data, null, 2).slice(10)})`);
}
//# sourceMappingURL=stream-utils.node.js.map